#include "SimHost.h"
#include <osg/Notify>

#include <OpenThreads/Thread>

// System includes
#include <cassert>

#ifdef _WIN32
#include <process.h>
#define getpid() _getpid()
#else
#include <unistd.h>
#endif 

// App includes


SimHost::SimHost()
{
	OSG_ALWAYS<<"SimHost instantiated."<<std::endl;
	numberofChannels = 6;
}

SimHost::~SimHost()
{
	//_channelWorker.clear();
}

void SimHost::initialize()
{
	done = false;
	int numThreads = numberofChannels+1;	// #channels + one main thread
	OpenThreads::Barrier syncBarrier;

	// Create IG Connectors
	OSG_ALWAYS<<"Creating IG Connectors..."<<std::endl;
	for(int i=0; i<numberofChannels; ++i)
	{
		osg::ref_ptr<ThreadedWorker> threadObject = new ThreadedWorker();
		osg::ref_ptr<IGConnector> IGCon = new IGConnector(numberofChannels, _frameBarrier);
		threadObject->setThreadWorker(IGCon);
		_threadObjects.push_back(threadObject);
		_IGConnectors.push_back(IGCon);
	}
	OSG_ALWAYS<<"...done."<<std::endl;

	// Setup Threading
	OSG_ALWAYS << "Root Thread ID: " << getpid() << std::endl;

	OSG_ALWAYS<<OpenThreads::GetNumberOfProcessors()<<" CPUs found, using "<<(numThreads)<<" of them concurrently."<<std::endl;
	OpenThreads::Thread::SetConcurrency( numThreads );	// Only supported by pthread implementation, otherwise ignored

	OpenThreads::Thread::Init();
	
}

void SimHost::run()
{
	// Launch threads
    for(int i=0;i<static_cast<int>(_threadObjects.size());++i)
	{
		int status;
		
		//thread->setStackSize(1024*256);
		status = _threadObjects[i]->start();
		assert(status == 0);
	}

	// Enter the SimHost main loop
	while(!done)
	{
		// Wait until all Channels have sent an SOF packet
		_frameBarrier.block(numberofChannels+1);  // Sync the main and all IG threads.
		OSG_ALWAYS<<"All IGConnectors have recieved a SOF packet: Starting new Frame"<<std::endl;
	}
	
	// Signal threads to finish
    for(int i=0;i<static_cast<int>(_threadObjects.size());++i)
	{
		//_threadObjects[i]->setDone(true);	// non blocking
		_threadObjects[i]->cancel();	// Blocking
	}
}